﻿using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace UltraGridRowsDragAndDrop
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ultraGrid1.InitializeLayout += new Infragistics.Win.UltraWinGrid.InitializeLayoutEventHandler(ultraGrid_InitializeLayout);
            ultraGrid2.InitializeLayout += new Infragistics.Win.UltraWinGrid.InitializeLayoutEventHandler(ultraGrid_InitializeLayout);
            ultraGrid1.SelectionDrag += new System.ComponentModel.CancelEventHandler(ultraGrid_SelectionDrag);
            ultraGrid2.SelectionDrag += new System.ComponentModel.CancelEventHandler(ultraGrid_SelectionDrag);
            ultraGrid1.DragOver += new DragEventHandler(ultraGrid_DragOver);
            ultraGrid2.DragOver += new DragEventHandler(ultraGrid_DragOver);
            ultraGrid1.DragDrop += new DragEventHandler(ultraGrid_DragDrop);
            ultraGrid2.DragDrop += new DragEventHandler(ultraGrid_DragDrop);

            var dataTable = GetData();
            ultraGrid1.DataSource = dataTable;
            ultraGrid2.DataSource = dataTable;
        }

        private void ultraGrid_DragDrop(object sender, DragEventArgs e)
        {
            var grid = sender as UltraGrid;
            int dropIndex = 0;

            // Get the position on the grid where the dragged row(s) are to be dropped
            // Get the grid coordinates of the row (the drop zone)
            var elementOver = grid.DisplayLayout.UIElement.ElementFromPoint(grid.PointToClient(new Point(e.X, e.Y)));

            // Get the row that is the drop zone/or where the dragged row is to be dropped
            var rowOver = elementOver.GetContext(typeof(UltraGridRow), true) as UltraGridRow;

            if (rowOver != null)
            {
                dropIndex = rowOver.Index;    //index/position of drop zone in grid
    
                // Get the dragged row(s)which are to be dragged to another position in the grid
                var selectedRows = e.Data.GetData (typeof(SelectedRowsCollection)) as SelectedRowsCollection;

                // Get the count of selected rows and drop each starting at the dropIndex
                foreach (UltraGridRow aRow in selectedRows)
                {
                    // Move the selected row(s) to the drop zone
                    grid.Rows.Move(aRow, dropIndex);
                }
            }
        }

        private void ultraGrid_DragOver(object sender, DragEventArgs e)
        {
            var grid = sender as UltraGrid;
            e.Effect = DragDropEffects.Move;
            Point pointInGridCoords = grid.PointToClient(new Point(e.X, e.Y));

            if (pointInGridCoords.Y < 20)
            {
                // Scroll Up
                grid.ActiveRowScrollRegion.Scroll(RowScrollAction.LineUp);
            }
            else if (pointInGridCoords.Y > grid.Height - 20)
            {
                // Scroll Down
                grid.ActiveRowScrollRegion.Scroll(RowScrollAction.LineDown);
            }
        }

        private void ultraGrid_SelectionDrag(object sender, System.ComponentModel.CancelEventArgs e)
        {
            var grid = sender as UltraGrid;
            grid.DoDragDrop(grid.Selected.Rows, DragDropEffects.Move);
        }

        private void ultraGrid_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            var grid = sender as UltraGrid;

            grid.AllowDrop = true;
            grid.DisplayLayout.Override.AllowUpdate = DefaultableBoolean.False;
            grid.DisplayLayout.Override.AllowDelete = DefaultableBoolean.False;
            grid.DisplayLayout.Override.SelectTypeCell = SelectType.None;
            grid.DisplayLayout.Override.CellClickAction = CellClickAction.RowSelect;
            grid.DisplayLayout.Override.SelectTypeRow = SelectType.ExtendedAutoDrag;
        }

        private DataTable GetData(int rows = 25)
        {
            DataTable table = new DataTable("Table1");

            table.Columns.Add("Boolean column", typeof(bool));
            table.Columns.Add("Integer column", typeof(int));
            table.Columns.Add("DateTime column", typeof(DateTime));
            table.Columns.Add("String column", typeof(string));

            for (int i = 0; i < rows; i++)
            {
                DataRow row = table.NewRow();
                row["Boolean column"] = i % 2 == 0 ? true : false;
                row["Integer column"] = i;
                row["String column"] = "text";
                row["DateTime column"] = DateTime.Today.AddDays(i);
                table.Rows.Add(row);
            }

            return table;
        }
    }
}
